clear all
clc
echo on
%
% This program is genreated to illustrate h-infinity controller
% design and d-k iteration used for highly manuverable aircraft HiMAT
%
%% Model of NASA's HiMAT aircraft
% For illustration purposes, let's use a four-state model of the
% longitudinal dynamics of the HiMAT aircraft.
%
% This model has two control inputs:
%
% * Elevon deflection
% * Canard deflection
%
% It also has two measured outputs:
%
% * Angle of attack alpha
% * Pitch angle theta
%
echo off
ag =[-0.0226   -36.6000   -18.9000   -32.1000
      0        -1.9000     0.9830     0
      0.0123    -11.7000   -2.6300     0
      0          0          1.0000     0 ];
bg = [ 0         0
      -0.4140    0
      -77.8000   22.4000
       0         0];
cg = [ 0     57.3000  0   0
       0       0      0   57.3000];
dg = [0   0;
      0   0];
G=ss(ag,bg,cg,dg);
G.InputName = {'elevon','canard'};
G.OutputName = {'alpha','theta'};
clf
plant_poles=pole(G), plant_zeros=zero(G)
step(G), title('Open-loop step response of HIMAT aircraft')

echo on
pause % press a key to continue
clc
% Creating an interconnection for plant model and
% specifying performance weighting function.
s=zpk('s'); % Laplace variable s
WDEL1 = 50*(s+100)/(s+1e4);
WP1 = 0.5*(s+3)/(s+0.03);
WDEL = append(WDEL1,WDEL1);
WP = [WP1 0;0 WP1];
echo off
systemnames = 'G WDEL WP';
inputvar = '[pert(2); dist(2); control(2)]';
outputvar = '[ WDEL ; WP; G + dist ]';
input_to_G = '[ control + pert ]';
input_to_WDEL = '[ control ]';
input_to_WP = '[ G + dist ]';
sysoutname = 'himat_ic';
cleanupsysic = 'yes';
sysic

bode(G(1,1),G(2,2))
d=legend('G(1,1):elevon to alpha', 'G(2,2):canard to theta')

echo on
pause % press a key to continue
clc
%
% The next step is to design an H-infinity control law for PLANT
%
% In this example, the system interconnection structure is
% himat_ic, with 2 measurements, 2 controls.
[k1,clp1,gamma1,info1]=hinfsyn(himat_ic,2,2,'Display','on');

omega=logspace(-1,5,500);
sig_g1=sigma(clp1,omega);
semilogx(omega,sig_g1(1,:))

 
echo on
pause % press a key to continue
clc
%
% The closed loop system is neither robust stable nor robustly
% performed. This can be visualized using robuststab and robustperf
% functions as in previous example. We like to use mu-synthesis to design a
% controller which is robust in performance.
%
% First generate the uncertain system structure.
delta = ultidyn('delta',[2 2]);
himat_pert=lft(delta,himat_ic);
clpm1=lft(himat_pert,k1);
% 
% Next, we perform a mu-synthesis to see if the specs can be met
% robustly when taking into account the modeling errors (uncertainty
% "delta"). We use the command "dksyn" to perform the synthesis and set
% the frequency grid used for mu-analysis and the number of D-K
% iterations with "dkitopts".
%
fmu = logspace(-2,4,60);
opt = dkitopt('FrequencyVector',fmu,'NumberofAutoIterations',5,...
'DisplayWhileAutoIter','on');
[kmu,clpmu,bnd,dkinfo] = dksyn(himat_pert,2,2,opt);

%   
%   The order of controller is high, use a balance realization
%   and a truncation to make it small. To find the proper reduction order
%   plot the hankel norms

hankelsv(kmu)

%
%   From the figure it is clear that the proper reduction order is 10
%
[kmu10,info_kmu10] = reduce(kmu,10);
clpmu10=lft(himat_pert,kmu10);

% Analyse robust performance
%
[perfmarg1,perfmargunc1,rep1,inf1] = robustperf(clpm1);
[perfmarg2,perfmargunc2,rep2,inf2] = robustperf(clpmu);
[perfmarg3,perfmargunc3,rep3,inf3] = robustperf(clpmu10);
semilogx(inf1.MussvBnds(:,1),'b',inf2.MussvBnds(:,1),':r',...
inf3.MussvBnds(:,1),'--g')
xlabel('Frequency (rad/sec)');
ylabel('Mu upper/lower bounds');
title('Robust Performance Mu Plot');
legend('hinf', 'mu' , 'mu reduced')

pause % press a key to continue
clc
P1 = usample(clpm1,5); % 5 random samples of the plant
P2 = usample(clpmu,5); % 5 random samples of the plant
P3 = usample(clpmu10,5); % 5 random samples of the plant

figure(2)
impulse(P1(2,2),0.1)
legend('H_\infty')
figure(3)
impulse(P2(2,2),':r',0.1)
legend('\mu synthesis')
figure(4)
impulse(P3(2,2),'--m',0.1)
legend('\mu reduced')
